//Code by Donald Bell (makerprojectlab.com) and Fuzzy Wobble (fuzzywobble.com)
//Created for interaction with a hellodrinkbot.com cocktail robot.
//March 2020

#include "Adafruit_Keypad.h"
#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266HTTPClient.h>
#define RELAY_PIN 5
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1:
#define LED_PIN     3

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT  32

// NeoPixel brightness, 0 (min) to 255 (max)
#define BRIGHTNESS 200

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_RGBW + NEO_KHZ800);
ESP8266WiFiMulti WiFiMulti;

const byte ROWS = 4; // rows
const byte COLS = 4; // columns
//define the symbols on the buttons of the keypads
char keys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

//pins for feather
byte rowPins[ROWS] = {0, 16, 2, 4}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {14, 12, 13, 15}; //connect to the column pinouts of the keypad

/*
   SEE THE DIAGRAM TO UNDERSTAND ROW/COLUMN ASSIGNMENT ABOVE
*/

//initialize an instance of class NewKeypad
Adafruit_Keypad customKeypad = Adafruit_Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS);


//char KEYPAD_INPUT[4] = "XXXX"; //why do we have to make it a lenght of 5, and not 4?
char KEYPAD_INPUT[4];
int KEYPAD_INPUT_INDEX = 0;

#define NUM_DRINKS 4
//change following numbers to change keypad codes
String DRINKS[NUM_DRINKS] = {"1316", "4455", "8151", "4415"};
//also change numbers in the 'void code_entered' section to match

//----------------------------------------------------------
void setup() {
  //  pinMode(LED_PIN, OUTPUT);
  pinMode(RELAY_PIN, OUTPUT);
  Serial.begin(9600);
  customKeypad.begin();
  clear_code();
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(200); // Set BRIGHTNESS (max = 255)
  // ESP8266 takes a few moments to boot, so we give it a few seconds
  for (uint8_t t = 4; t > 0; t--)
  {
    Serial.printf("SETUP WAIT %d...\n", t);
    Serial.flush();
    colorWipe(strip.Color(  0,   0,   0, 255), 100); // True white (not RGB white)
    delay(1000);
  }

  // Add the WiFi access point information
  WiFiMulti.addAP("replace with server name", "replace with your password");
  //WiFiMulti.addAP("HelloDrinkbot", "imathirstygirl"); //defaults for Hello Drinkbot

}




//----------------------------------------------------------
void loop() {
  // put your main code here, to run repeatedly:
  // wait for WiFi connection
  if ((WiFiMulti.run() == WL_CONNECTED))
  {
    customKeypad.tick();

    while (customKeypad.available()) {
      keypadEvent e = customKeypad.read();
      if (e.bit.EVENT == KEY_JUST_PRESSED) {
        Serial.print((char)e.bit.KEY);
        char key = (char)e.bit.KEY;
        if (key == '0' || key == '1' || key == '2' || key == '3' || key == '4' || key == '5' || key == '6' || key == '7' || key == '8' || key == '9') {
          //a key was entered
          Serial.print("key was entered"); Serial.println();
          KEYPAD_INPUT[KEYPAD_INPUT_INDEX] = key;
          KEYPAD_INPUT_INDEX++;
          if (KEYPAD_INPUT_INDEX >= 4) {
            code_entered();
          }
        }
        if (key == '*' || key == '#') {
          //star or pound was entered
          Serial.print("star or pound was entered"); Serial.println();
        }
        if (key == 'a' || key == 'A' || key == 'b' || key == 'B' || key == 'c' || key == 'C' || key == 'd' || key == 'D') {
          //a letter was entered
          Serial.print("letter entered"); Serial.println();
        }
      }

      //      Serial.print((char)e.bit.KEY);
      //      read_keypad();
      //      if(e.bit.EVENT == KEY_JUST_PRESSED) Serial.println(" pressed");
      //      else if(e.bit.EVENT == KEY_JUST_RELEASED) Serial.println(" released");
    }

    delay(10);
  }
  else
  {
    Serial.println("No WiFi");
    rainbowFade2White(3, 3, 1);
  }
}






//----------------------------------------------------------
void read_keypad() {
  keypadEvent e = customKeypad.read();
  Serial.print((char)e.bit.KEY);
  char key = (char)e.bit.KEY;
  if (key == '0' || key == '1' || key == '2' || key == '3' || key == '4' || key == '5' || key == '6' || key == '7' || key == '8' || key == '9') {
    //a key was entered
    Serial.print("key was entered"); Serial.println();
    KEYPAD_INPUT[KEYPAD_INPUT_INDEX] = key;
    KEYPAD_INPUT_INDEX++;
    if (KEYPAD_INPUT_INDEX >= 4) {
      code_entered();
    }
  }
  if (key == '*' || key == '#') {
    //star or pound was entered
    Serial.print("star or pound was entered"); Serial.println();
  }
  if (key == 'a' || key == 'A' || key == 'b' || key == 'B' || key == 'c' || key == 'C' || key == 'd' || key == 'D') {
    //a letter was entered
    Serial.print("letter entered"); Serial.println();
  }
}



//----------------------------------------------------------
void code_entered() {

  String code_4 = String(KEYPAD_INPUT[0]) + String(KEYPAD_INPUT[1]) + String(KEYPAD_INPUT[2]) + String(KEYPAD_INPUT[3]);

  bool drink_was_found = false;

  for (int i = 0; i < NUM_DRINKS; i++) {
    if (code_4 == DRINKS[i]) {
      drink_was_found = true;
      Serial.println("YAY WE FOUND A MATCH. MAKE A DRINK!");
      if (code_4 == "4455") { //change this to match a code in definitions
        //make cocktail 1
        clear_code();
        digitalWrite(RELAY_PIN, HIGH);
        HTTPClient http;
        
        // key for http ordering
        //66 = 2oz
        //24 = 3/4 oz
        //8 = 1/4 oz
        //If you assign multiple pumps to the same liquor,
        //all of the assigned pumps will pumps the ordered amount (not automatically divide it among the pumps).
        //So be sure to divide the amount being sent by the number of pumps assigned to that ingredient
        //or you'll have twice the amount of that ingredient.
        //That said, assigning multiple pumps to your main ingredient is a smart way to get drinks out faster.

        http.begin("http://10.0.0.1/ws/drink/12?booze7=33&booze16=24&booze80=8");
        int httpCode = http.GET();
        http.end();
        colorWipe(strip.Color(255,   0,   0)     , 50); // Green, last number is delay
        pulseWhite(40); //number adjusts delay between pulses
        rainbowFade2White(2, 20, 15); //speed,rainbow loops,white loops
        delay(500);
        colorWipe(strip.Color(  0,   0,   0, 255), 400); // True white (not RGB white), last number is delay
        delay(500);
        digitalWrite(RELAY_PIN, LOW);
      }

      if (code_4 == "4415") {
        //make cocktail 2
        clear_code();
        digitalWrite(RELAY_PIN, HIGH);
        HTTPClient http;
        // Be sure to change this IP address and port number
        //  to match yours!!!
        http.begin("http://10.0.0.1/ws/drink/12?booze1=120&booze79=120");
        int httpCode = http.GET();
        http.end();
        colorWipe(strip.Color(255,   0,   0)     , 100); // Green
        pulseWhite(75);
        rainbowFade2White(2, 60, 10);
        delay(100);
        colorWipe(strip.Color(  0,   0,   0, 255), 50); // True white (not RGB white)
        digitalWrite(RELAY_PIN, LOW);
      }

      if (code_4 == "8151") {
        //make cocktail 3
        clear_code();
        digitalWrite(RELAY_PIN, HIGH);
        HTTPClient http;
        // Be sure to change this IP address and port number
        //  to match yours!!!
        http.begin("http://10.0.0.1/ws/drink/12?booze1=120&booze79=120");
        int httpCode = http.GET();
        http.end();
        colorWipe(strip.Color(255,   0,   0)     , 100); // Green
        pulseWhite(75);
        rainbowFade2White(2, 60, 10);
        delay(1000);
        colorWipe(strip.Color(  0,   0,   0, 255), 100); // True white (not RGB white)
        digitalWrite(RELAY_PIN, LOW);
      }

      if (code_4 == "1316") {
        //make cocktail 4
        clear_code();
        digitalWrite(RELAY_PIN, HIGH);
        HTTPClient http;
        // Be sure to change this IP address and port number
        //  to match yours!!!
        http.begin("http://10.0.0.1/ws/drink/12?booze1=120&booze79=120");
        int httpCode = http.GET();
        http.end();
        colorWipe(strip.Color(255,   0,   0)     , 100); // Green
        pulseWhite(75);
        rainbowFade2White(2, 60, 10);
        delay(100);
        colorWipe(strip.Color(  0,   0,   0, 255), 50); // True white (not RGB white)
        digitalWrite(RELAY_PIN, LOW);
      }

    }
  }
  if (drink_was_found == false) {
    Serial.println("NO MATCH FOUND :(");
    clear_code();

    colorWipe(strip.Color(  0, 255,   0)     , 50); // Red
    delay(2000);
    colorWipe(strip.Color(  0,   0,   0, 255), 50); // True white (not RGB white)
  }

}



//----------------------------------------------------------
void clear_code() {
  KEYPAD_INPUT_INDEX = 0;
  KEYPAD_INPUT[0] = 'X';
  KEYPAD_INPUT[1] = 'X';
  KEYPAD_INPUT[2] = 'X';
  KEYPAD_INPUT[3] = 'X';
}
// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.

void colorWipe(uint32_t color, int wait) {
  for (int i = 0; i < strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}

void whiteOverRainbow(int whiteSpeed, int whiteLength) {

  if (whiteLength >= strip.numPixels()) whiteLength = strip.numPixels() - 1;

  int      head          = whiteLength - 1;
  int      tail          = 0;
  int      loops         = 100;
  int      loopNum       = 0;
  uint32_t lastTime      = millis();
  uint32_t firstPixelHue = 0;

  for (;;) { // Repeat forever (or until a 'break' or 'return')
    for (int i = 0; i < strip.numPixels(); i++) { // For each pixel in strip...
      if (((i >= tail) && (i <= head)) ||     //  If between head & tail...
          ((tail > head) && ((i >= tail) || (i <= head)))) {
        strip.setPixelColor(i, strip.Color(0, 0, 0, 255)); // Set white
      } else {                                             // else set rainbow
        int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
        strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
      }
    }

    strip.show(); // Update strip with new contents
    // There's no delay here, it just runs full-tilt until the timer and
    // counter combination below runs out.

    firstPixelHue += 40; // Advance just a little along the color wheel

    if ((millis() - lastTime) > whiteSpeed) { // Time to update head/tail?
      if (++head >= strip.numPixels()) {     // Advance head, wrap around
        head = 0;
        if (++loopNum >= loops) return;
      }
      if (++tail >= strip.numPixels()) {     // Advance tail, wrap around
        tail = 0;
      }
      lastTime = millis();                   // Save time of last movement
    }
  }
}

void pulseWhite(uint8_t wait) {
  for (int j = 40; j < 256; j++) { // Ramp up from 0 to 255
    // Fill entire strip with white at gamma-corrected brightness level 'j':
    strip.fill(strip.Color(0, 0, 0, strip.gamma8(j)));
    strip.show();
    delay(wait);
  }

}

void rainbowFade2White(int wait, int rainbowLoops, int whiteLoops) {
  int fadeVal = 0, fadeMax = 100;

  // Hue of first pixel runs 'rainbowLoops' complete loops through the color
  // wheel. Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to rainbowLoops*65536, using steps of 256 so we
  // advance around the wheel at a decent clip.
  for (uint32_t firstPixelHue = 0; firstPixelHue < rainbowLoops * 65536;
       firstPixelHue += 256) {

    for (int i = 0; i < strip.numPixels(); i++) { // For each pixel in strip...

      // Offset pixel hue by an amount to make one full revolution of the
      // color wheel (range of 65536) along the length of the strip
      // (strip.numPixels() steps):
      uint32_t pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());

      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the three-argument variant, though the
      // second value (saturation) is a constant 255.
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue, 255,
                                           255 * fadeVal / fadeMax)));
    }

    strip.show();
    delay(wait);

    if (firstPixelHue < 65536) {                             // First loop,
      if (fadeVal < fadeMax) fadeVal++;                      // fade in
    } else if (firstPixelHue >= ((rainbowLoops - 1) * 65536)) { // Last loop,
      if (fadeVal > 0) fadeVal--;                            // fade out
    } else {
      fadeVal = fadeMax; // Interim loop, make sure fade is at max
    }
  }

  for (int k = 0; k < whiteLoops; k++) {
    for (int j = 0; j < 256; j++) { // Ramp up 0 to 255
      // Fill entire strip with white at gamma-corrected brightness level 'j':
      strip.fill(strip.Color(0, 0, 0, strip.gamma8(j)));
      strip.show();
    }
    delay(100); // Pause 1 second
    for (int j = 255; j >= 0; j--) { // Ramp down 255 to 0
      strip.fill(strip.Color(0, 0, 0, strip.gamma8(j)));
      strip.show();
    }
  }

  delay(500); // Pause 1/2 second
}
